package tests_test

import (
	"os"
	"os/exec"
	"path/filepath"
	"testing"

	"git.akyoto.dev/cli/q/src/build"
	"git.akyoto.dev/go/assert"
)

func TestPrograms(t *testing.T) {
	var tests = []struct {
		Name             string
		ExpectedOutput   string
		ExpectedExitCode int
	}{
		{"successive-calls.q", "", 9},
	}

	for _, test := range tests {
		t.Run(test.Name, func(t *testing.T) {
			run(t, filepath.Join("programs", test.Name), test.ExpectedOutput, test.ExpectedExitCode)
		})
	}
}

// run builds and runs the file to check if the output matches the expected output.
func run(t *testing.T, name string, expectedOutput string, expectedExitCode int) {
	b := build.New(name)
	assert.True(t, len(b.Executable()) > 0)

	t.Run("Compile", func(t *testing.T) {
		result, err := b.Run()
		assert.Nil(t, err)

		err = result.Write(b.Executable())
		assert.Nil(t, err)

		stat, err := os.Stat(b.Executable())
		assert.Nil(t, err)
		assert.True(t, stat.Size() > 0)
	})

	t.Run("Output", func(t *testing.T) {
		cmd := exec.Command(b.Executable())
		output, err := cmd.Output()
		exitCode := 0

		if err != nil {
			exitError, ok := err.(*exec.ExitError)

			if !ok {
				t.Fatal(exitError)
			}

			exitCode = exitError.ExitCode()
		}

		assert.Equal(t, exitCode, expectedExitCode)
		assert.DeepEqual(t, string(output), expectedOutput)
	})
}
